// AirPong
// Clarence Pearson

#include <stdlib.h>
#include <stdio.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include "SDL.h"
// include leadpipe headers
#include "cengine.h"
// game scene header
#include  "game.h"

// key defs
bool key_right;
bool key_left;

// CAirPong class
class CAirPong : public CEngine {
	public:
		// call constructor
		CAirPong(char *name, int w, int h, int bpp, int fs);
		// destructor
		~CAirPong() { delete game; }
		// render scene
		virtual void RenderScene();
		// keyboard handler
		virtual void HandleKeyboard(SDL_Event *event);
		// mouse movement handler
		virtual void HandleMouseMovement(SDL_Event *event);
		// get key state (faster than SDL events)
		void GetKeyState();
	private:
		CGame *game;
};

// CAirPong constructor
CAirPong::CAirPong(char *name, int w, int h, int bpp, int fs) : CEngine(name,w,h,bpp,fs) {
	// create new test scene
	game = new CGame(this);
	// don't show mouse
	SDL_ShowCursor(SDL_DISABLE);
}

// render scene
void CAirPong::RenderScene() {
	// render scene
	game->Update();
	game->Render();
	// update key state
	GetKeyState();
}

// overload keyboard handler to allow movement
void CAirPong::HandleKeyboard(SDL_Event *event) {
	// check what key is pressed
    switch(event->key.keysym.sym) {
		// move bat right and left
		case SDLK_ESCAPE:
			// flip menu flag
			if(game->menu->running) game->menu->running = false;
			else game->menu->running = true;
			break;
		case SDLK_1:
			// is menu running?
			if(game->menu->running) {
				// new game
				game->ball->Reset();
				game->pbat->Reset();
				game->cbat->Reset();
				game->game_vars->player_score = 0;
				game->game_vars->computer_score = 0;
				game->menu->running = false;
			}
			break;
		case SDLK_2:
			// quit
			if(game->menu->running) {
				engine_run = 0;
			}
			break;
		default:
			break;
    }
	// leave
	return;
}

// handle mouse movement
void CAirPong::HandleMouseMovement(SDL_Event *event) {
	return;
}

// get key state
void CAirPong::GetKeyState() {
	Uint8 *keys;

	keys = SDL_GetKeyState(NULL);

	if(keys[SDLK_RIGHT]) {
		key_right = true;
	}

	if(keys[SDLK_LEFT]) {
		key_left = true;
	}
}

// main
int main(int argc, char *argv[]) {
    // create an instance of the engine
    CAirPong *pong = new CAirPong("AirPong",640,480,16,-1);

    // check for error
    if(pong == NULL) {
        fprintf(stderr, "Error creating CEngine Object.\n");
        exit(1);
    }

    // run engine...pass scene
    pong->RunCycle();

    // done
    delete pong;
    return 0;
}
