// AirPong
// Clarence Pearson

#include <GL/gl.h>
#include <GL/glu.h>
#include "game.h"
#include "airpong.h"
#include "objects.h"
#include "cmenu.h"
#include "cparticle.h"
// leadpipe headers
#include "cengine.h"

// constructor
CGame::CGame(CEngine *window) : CScene(window) {
	font = new CFont;
	// load font
	font->Load("data/font.bmp");

	// set arena bounds
	// x bounds
	arena[0] = -0.65;
	arena[1] = 0.65;
	// z bounds
	arena[2] = -0.5;
	arena[3] = 1.5;

	// new ball
	ball = new CBall();
	// new player bat
	pbat = new CPlayer();
	// new computer
	cbat = new CComputer();
	// new game vars
	game_vars = new CGameVars();
	// new menu
	menu = new CMenu();
	// new particles
	particles = new CParticle[200];
	// init particles
	for(int c = 0; c < 200; c++) {
		particles[c].init(1.0f, 0.3f, 0.0f);
	}
}

// destructor
CGame::~CGame() {
	// free font
	delete font;
	// free timer
	delete timer;
	// delete ball
	delete ball;
	// delete player
	delete pbat;
	// delete computer
	delete cbat;
	// free vars
	delete game_vars;
	// free menu
	delete menu;
	// free particles
	delete[] particles;
}

// Render scene
void CGame::Render() {
	// reset matrix
	glLoadIdentity();

	// give tilted look
	gluLookAt(0, 1.5, 2.25f, 0, -0.3f, 0.5, 0, 1, 0);

	// draw arena
	glColor3f(0.5, 0.0f, 0.0f);
	glBegin(GL_LINES);
		// bottom
		glVertex3f(arena[0], 0, arena[2]);
		glVertex3f(arena[1], 0, arena[2]);
		// right
		glVertex3f(arena[1], 0, arena[2]);
		glVertex3f(arena[1], 0, arena[3]);
		// top
		glVertex3f(arena[1], 0, arena[3]);
		glVertex3f(arena[0], 0, arena[3]);
		// left
		glVertex3f(arena[0], 0, arena[3]);
		glVertex3f(arena[0], 0, arena[2]);
	glEnd();

	// draw bat
	pbat->Draw();
	cbat->Draw();

	// draw ball
	ball->Draw();
	
	// draw particles
	for(int c = 0; c < 200; c++) {
		particles[c].draw();
	}

	// enable texturing
	glEnable(GL_TEXTURE_2D);
	// set ortho projection
	SetOrtho();
	// print
	glColor3f(1.0f, 1.0f, 1.0f);
	font->SetPos(0.0f, 0.0f);
	font->Print(ITAL,"AirPong %d.%d.%d", MAJOR, MINOR, REV);
	font->SetPos(0.0f, 15.0f);
	font->Print(NORM, "FPS: %d", timer->GetFPS());
	font->SetPos(0.0, 30.0f);
	font->Print(NORM, "Player: %d", game_vars->player_score);
	font->SetPos(0.0, 45.0f);
	font->Print(NORM, "Computer: %d", game_vars->computer_score);
	// is menu running
	if(menu->running) {
		font->SetPos(210.0f, 160.0f);
		font->Print(NORM, "%s", menu->option1);
		font->SetPos(210.0f, 180.0f);
		font->Print(NORM, "%s", menu->option2);
	}
	// unset ortho
	UnSetOrtho();
	// disable texturing
	glDisable(GL_TEXTURE_2D);
}

// update scene
void CGame::Update() {
	// timer update
	timer->Update();
	// limit fps to 100
	timer->Limit(100);
	// if menu is not running
	if(!menu->running) {
		// update ball
		ball->Update(timer->GetDeltaTime(), arena[0], arena[1]);
		// update computer
		cbat->Update(timer->GetDeltaTime(), arena[0], arena[1], ball);

		// check key states
		if(key_left) {
			key_left = false;
			pbat->MoveL(timer->GetDeltaTime(), arena[0]);
		} else if(key_right) {
			key_right = false;
			pbat->MoveR(timer->GetDeltaTime(), arena[1]);
		}

		// check for collisions
		if(Collision(pbat, ball)) {
			// negate z velocity
			ball->v_vel.z = -ball->v_vel.z;
			// set x velocity
			ball->v_vel.x = ball->v_vel.x + (ball->v_pos.x - (pbat->v_pos.x + (pbat->w / 2))) * .9;
		}

		// check for collisions
		if(Collision(cbat, ball)) {
			// negate z velocity
			ball->v_vel.z = -ball->v_vel.z;
			// set x velocity
			ball->v_vel.x = ball->v_vel.x + (ball->v_pos.x - (cbat->v_pos.x + (cbat->w / 2))) * .9;
		}

		// check for score
		if(ball->v_pos.z < arena[2]) {
			// reset objects
			ball->Reset();
			pbat->Reset();
			cbat->Reset();
			// player scored
			game_vars->player_score++;
			// check for win
			if(game_vars->player_score == 10) {
			// run menu
			// reset score
			menu->running = true;
			game_vars->player_score = 0;
			game_vars->computer_score = 0;
			}
		}
		// check for score
		if(ball->v_pos.z > arena[3] - ball->h) {
			// reset objects
			ball->Reset();
			pbat->Reset();
			cbat->Reset();
			// player scored
			game_vars->computer_score++;
			// check for win
			if(game_vars->computer_score == 10) {
				// run menu
				// reset score
				menu->running = true;
				game_vars->player_score = 0;
				game_vars->computer_score = 0;
			}
		}
		// update particles
		for(int c = 0; c < 200; c++) {
			particles[c].update(ball, timer->GetDeltaTime());
		}
	}

}
