// Clarence Pearson
// LeadPipe OpenGL Engine

// CCamera Class
// basic camera class...nothing fancy or special

#include <GL/gl.h>
#include <GL/glu.h>
#include <math.h>
#include "ccamera.h"

// constructor
CCamera::CCamera() {
	// set vectors to initial values
	v_pos.x = 0.0; v_pos.y = 0.0; v_pos.z = 0.0;
	// position vector
	v_view.x = 0.0; v_view.y = 1.0; v_view.z = 0.5;
	// view vector
	v_up.x = 0.0; v_up.y = 0.0; v_up.z = 1.0;
	// up vector
}

// position camera
void CCamera::Position(float px, float py, float pz, float vx, float vy, float vz, float ux, float uy, float uz) {
	// set vectors
	v_pos.x = px; v_pos.y = py; v_pos.z = pz;
	v_view.x = vx; v_view.y = vy; v_view.z = vz;
	v_up.x = ux; v_up.y = uy; v_up.z = uz;
}

// look
void CCamera::Look() {
	// gluLookAt camera vectors
	gluLookAt(v_pos.x,v_pos.y,v_pos.z,v_view.x,v_view.y,v_view.z,v_up.x,v_up.y,v_up.z);
}

// rotate camera
void CCamera::Rotate(float angle, float x, float y, float z) {
	// view vector and new view vector
	CVector3 vVector;
	CVector3 vNewView;

	// sin and cosine of angle
	float cos_theta = (float)cos(angle);
	float sin_theta = (float)sin(angle);

	// get direction that camera is facing
	vVector.x = v_view.x - v_pos.x;
	vVector.y = v_view.y - v_pos.y;
	vVector.z = v_view.z - v_pos.z;

	// get new view x position
	vNewView.x = (cos_theta + (1 - cos_theta) * x * x) * vVector.x;
	vNewView.x += ((1 - cos_theta) * x * y - z * sin_theta) * vVector.y;
	vNewView.x += ((1 - cos_theta) * x * z + y * sin_theta) * vVector.z;

	// get new view y position
	vNewView.y = ((1 - cos_theta) * x * y + z * sin_theta) * vVector.x;
	vNewView.y += (cos_theta + (1 - cos_theta) * y * y) * vVector.y;
	vNewView.y += ((1 - cos_theta) * y * z - x * sin_theta) * vVector.z;

	// get new view z position
	vNewView.z = ((1 - cos_theta) * x * z - y * sin_theta) * vVector.x;
	vNewView.z += ((1 - cos_theta) * y * z + x * sin_theta) * vVector.y;
	vNewView.z += (cos_theta + (1 - cos_theta) * z * z) * vVector.z;

	// add the new view vector to our position
	v_view = v_pos + vNewView;
}

// strafe camera
void CCamera::Strafe(float speed, float dt) {
	// get strafe vector by normalizing cross product
	// of view - position and up vectors
	CVector3 v_strafe = Normalize(Cross(v_view - v_pos, v_up));

	// speed = speed * delta_time
	float k_speed = speed * dt;

	// add strafe vector to position
	v_pos.x += v_strafe.x * k_speed;
	v_pos.z += v_strafe.z * k_speed;

	// add strafe vector to view
	v_view.x += v_strafe.x * k_speed;
	v_view.z += v_strafe.z * k_speed;
}

// mouse look
void CCamera::MouseLook(float x, float y, float midx, float midy) {
	// angle of lookin up and down
	float angle_y = 0.0f;
	// angle of rotation around y axis
	float angle_z = 0.0f;
	// current rotation
	static float current_x = 0.0f;


	// get the direction the mouse moved in
	angle_y = (float)((midx - x)) / 700.0f;
	angle_z = (float)((midy - y)) / 700.0f;

	// keep track of current rotation
	current_x -= angle_z;

	// keep from rotating 360 degrees
	if(current_x > 1.0f) {
		current_x = 1.0f;
	} else if(current_x < -1.0f) {
		current_x = -1.0f;
	} else {
		// find axis to rotate around
		CVector3 v_axis = Cross(v_view - v_pos, v_up);
		// normalize it
		v_axis = Normalize(v_axis);

		// rotate view around axis
		Rotate(angle_z, v_axis.x, v_axis.y, v_axis.z);
		Rotate(angle_y, 0, 1, 0);
	}
	
	return;
}

// move camera forward and backward
void CCamera::Move(float speed, float dt) {
	// initial vector for view
	// get direction that camera is facing
	CVector3 vVector = v_view - v_pos;
	vVector = Normalize(vVector);

	// speed = speed * delta_time
	float k_speed = speed * dt;

	// add acceleration
	v_pos.x += vVector.x * k_speed;
	v_pos.z += vVector.z * k_speed;

	v_view.x += vVector.x * k_speed;
	v_view.z += vVector.z * k_speed;
}
