// Clarence Pearson
// LeadPipe OpenGL Engine

// CEngine Class
// Wraps window, input, and objects together
// This is a base class that is based on COGLwin

#include "SDL.h"
#include "cengine.h"
#include "cscene.h"

// constructor
CEngine::CEngine(char *name,int w, int h, int bpp, int fs) : COGLWin(name,w,h,bpp,fs) {}

// run the engine
void CEngine::RunCycle() {
    // set engine_run = 1
    engine_run = 1;

    // loop until time to quit
    while(engine_run) {
        // handle events
        HandleEvent();

        // clear window
        ClearWin();
		
		// Render Scene
		RenderScene();

        // draw window
        DrawWin();
    }

    // good to go
    return;
}

// handle events
void CEngine::HandleEvent() {
    // SDL Event
    SDL_Event event;

    // poll for event
    while(SDL_PollEvent(&event)) {
        // determine event type
        switch(event.type) {
            case SDL_KEYDOWN:
                // handle keypress
                HandleKeyboard(&event);
                break;
            case SDL_MOUSEMOTION:
                // handle mouse movement
                HandleMouseMovement(&event);
                break;
            case SDL_MOUSEBUTTONDOWN:
                // handle mouse button
                HandleMouseButton(&event);
				break;
            case SDL_VIDEORESIZE:
                // resize window
                ResizeWindow(event.resize.w, event.resize.h);
                break;
            case SDL_QUIT:
                // close engine
                engine_run = 0;
                break;
            default:
                break;
        }
    }
    // good to go
    return;
}

// handle keyboard
void CEngine::HandleKeyboard(SDL_Event *event) {
    // check what key is pressed
    switch(event->key.keysym.sym) {
        case SDLK_ESCAPE:
            // quit
            engine_run = 0;
            break;
        default:
            break;
    }
    // leave
    return;
}

// handle mouse movement
void CEngine::HandleMouseMovement(SDL_Event *event) {
}

// handle mouse button
void CEngine::HandleMouseButton(SDL_Event *event) {
}
