// Clarence Pearson
// LeadPipe OpenGL Engine

// CFont Class
// Builds font and displays text

#include <GL/gl.h>
#include <GL/glu.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include "cfont.h"
#include "ctexture.h"

// constructor
CFont::CFont() {
	// create new texture
	font = new CTexture;
}

// destructor
CFont::~CFont() {
	// delete list
	glDeleteLists(fontlist, 256);
}

// load font texture
// return 1 on success
int CFont::Load(char *fn) {
	// load texture
	if(font->Load(fn, BMP)) {
		// build font
		Build();
		// return success
		return 1;
	} else {
		// error
		return -1;
	}
}

// build font
void CFont::Build() {
	// character coordinates
    float cx, cy;
    int loop;
	
	// make sure texturing is enabled
	glEnable(GL_TEXTURE_2D);

    // generate 256 list
    fontlist = glGenLists(256);

    // select font texture
    glBindTexture(GL_TEXTURE_2D, font->tex_id);

    // loop through
	for(loop=0; loop<256; loop++) {
	// get current positions
    	cx=1 - (float)(loop%16)/16.0f;
        cy=1 - (float)(loop/16)/16.0f;

        // compile new list
        glNewList(fontlist + (255 - loop), GL_COMPILE);
			glBegin(GL_QUADS);
			glTexCoord2f(cx - 0.0625f, cy);
				glVertex2i(0,16);
				glTexCoord2f(cx, cy);
            	glVertex2i(16,16);
                glTexCoord2f(cx, cy - 0.0625f);
                glVertex2i(16,0);
                glTexCoord2f(cx - 0.0625,cy-0.0625);
                glVertex2i(0,0);
			glEnd();
            glTranslated(10,0,0);
		glEndList();
	}
	
	// disable texturing
	glDisable(GL_TEXTURE_2D);

	// good to go
	return;
}

// set position
void CFont::SetPos(float nx, float ny) {
	x = nx; y = ny;
}

// print string
void CFont::Print(int set, const char *string, ...) {
	// text buffer
	char text[255];
	// list of arguments passed
	va_list args;

	// check to see if any text was passed
	if(string == NULL) {
		return;
	}

	// parse string for variables
	va_start(args, string);
		vsprintf(text, string, args);
	va_end(args);

	// check for font set
	if(set > 1) { set = 1; };

	// disable depth testing
	glDisable(GL_DEPTH_TEST);
	
	// reset matrix
	glLoadIdentity();
	// move 1 unit into screen
	glTranslatef(x, y, -1.0f);
	
	// bind font texture
	font->Bind();

	// get set from list
	glListBase(fontlist-32+(128*1));
	// call font list
	glCallLists(strlen(text), GL_UNSIGNED_BYTE, text);
	
	glPopMatrix();

	// enable depth testing and lighting
	glEnable(GL_DEPTH_TEST);

	// good to go
	return;
}
