// Clarence Pearson
// LeadPipe OpenGL Engine

// COGLWin Class
// Creates an OpenGL window using SDL
// Initializes OpenGL

#include <stdlib.h>
#include <stdio.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include "SDL.h"
#include "coglwin.h"

// constructor
// builds window
COGLWin::COGLWin(char *win_name, int win_width, int win_height, int win_bpp, int full_flag) {
    // initialize SDL
    if(SDL_Init(SDL_INIT_VIDEO) < 0) {
        // error initializing SDL
        // print error
        fprintf(stderr, "Error initializing SDL...exiting\n");
        // exit
        exit(1);
    }

    // set video flags
    v_flags  = SDL_OPENGL;
    v_flags |= SDL_GL_DOUBLEBUFFER;
    v_flags |= SDL_HWPALETTE;
	v_flags |= SDL_RESIZABLE;
    v_flags |= SDL_SWSURFACE;
    // check for fullscreen flag
    if(full_flag > 0) {
        v_flags |= SDL_FULLSCREEN;
    }

    // set double buffering
    SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );

    // set width, height, and bpp
    screen_w = win_width;
    screen_h = win_height;
    bpp = win_bpp;

    // create SDL surface
    screen = SDL_SetVideoMode(screen_w, screen_h, bpp, v_flags);

    // check for error
    if(!screen) {
        fprintf(stderr, "Error initializing SDL surface...exiting\n");
        // shutdown SDL
        SDL_Quit();
        // exit
        exit(1);
    }

    // set window caption
    SDL_WM_SetCaption(win_name, 0);

    // initialize OpenGL
    InitOGL();

    // resize the initial window
    ResizeWindow(screen_w, screen_h);
	
	// enable key repeat
	SDL_EnableKeyRepeat(100, SDL_DEFAULT_REPEAT_INTERVAL);

    // done
}

// destructor
COGLWin::~COGLWin() {
    // quit SDL gracefully
    SDL_Quit();
}

// initialize OpenGL
void COGLWin::InitOGL() {
    // enable texture mapping
    glEnable(GL_TEXTURE_2D);
    // enable smooth shading
    glShadeModel(GL_SMOOTH);
    // set background color to black
    glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
    // depth buffer setup
    glClearDepth(1.0f);
    // enables depth testing
    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LEQUAL);
    // enable blending
	glEnable( GL_BLEND );
	glBlendFunc( GL_SRC_ALPHA, GL_ONE );
    // set perspective
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

    // done
    return;
}

// Resize Window function
void COGLWin::ResizeWindow(int new_w, int new_h) {
    // height/width rotation ratio
    GLfloat ratio;

    // set screen w,h = new w,h
    screen_w= new_w; screen_h = new_h;

    // change video mode
    screen = SDL_SetVideoMode(screen_w, screen_h, bpp, v_flags);
    // check to see if surface was created
    if (!screen) {
        fprintf(stderr, "Could not get a surface after resize: %s\n", SDL_GetError());
        exit(1);
	}

    // set rotation ratio
    ratio = (GLfloat)(new_w / new_h);

    // set up viewport
    glViewport(0, 0, (GLsizei)new_w, (GLsizei)new_h);

    // change to projection matrix
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    // set perspective
    gluPerspective(45.0f, ratio, 0.1f, 100.0f);

    // change modelview matrix
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    // done
    return;
}

// the next functions clear and update the window
// clear should be called before any objects in the loop
// draw shout be called last
// like this:
//
// window.Clear();
// scene.Render();
// window.Draw();

// clear window
void COGLWin::ClearWin() {
    // clear the screen and depth buffer
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    // done
    return;
}

// draw window
void COGLWin::DrawWin() {
    // swap buffers
    SDL_GL_SwapBuffers();
    // done
    return;
}

