// Clarence Pearson
// LeadPipe OpenGL Engine

// Various Math functions

#include <math.h>
#include "cvector.h"
#include "lpmath.h"

// return the normal of a polygon
CVector3 PolyNormal(CVector3 vPoly[]) {
	// get two vectors from the polygon
	CVector3 v1 = Vector(vPoly[2], vPoly[0]);
	CVector3 v2 = Vector(vPoly[1], vPoly[0]);

	// get cross product
	CVector3 v_normal = Cross(v1, v2);

	// normalize it
	v_normal = Normalize(v_normal);
	
	// return it
	return v_normal;
}

// return the distance a plane is from the origin
// takes normal to plane and any point in plane
float PlaneDistance(CVector3 vnormal, CVector3 vpoint) {
	// distance variable
	float distance = 0;
	
	// negate dot product to get distance
	distance = -(Dot(vnormal, vpoint));
	
	// return it
	return distance;
}

// return true if line and plane intersect
// takes plane and line
bool IntersectedPlane(CVector3 vPlane[], CVector3 vLine[]) {
	// distances from the two points of the line
	float d1, d2;
	
	// get normal of plane
	CVector3 vnormal = PolyNormal(vPlane);
	
	// get distance from origin
	float o_distance = PlaneDistance(vnormal, vPlane[0]);
	
	// get distance = dot(normal, line) + distance from origin
	d1 = Dot(vnormal, vLine[0]) + o_distance;
	d2 = Dot(vnormal, vLine[1]) + o_distance;
	
	// check for intersection
	// if d1 * d2 < 0 then line intersects plane
	// b/c point are on different sides of plane
	if(d1 * d2 >= 0) {
		// no collision
		return false;
	}
	
	// else return true
	return true;
}

