// AirPong
// Clarence Pearson

// define classes for AirPong objects

#include <GL/gl.h>
#include <GL/glu.h>
#include "objects.h"

// ball constructor
CBall::CBall() {
	// set velocity
	v_vel.x = 0.0f;
	v_vel.z = -1.7f;

	// set position
	v_pos.x = 0.09f;
	v_pos.z = 0.75f;

	// set size
	w = 0.012f;
	h = 0.01f;
}

// update ball
void CBall::Update(float dt, float x1, float x2) {
	// add velocity * dt
	v_pos.x += v_vel.x * dt;
	v_pos.z += v_vel.z * dt;

	// check for arena collisions
	if((v_pos.x < x1) || (v_pos.x > x2 - h)) {
		// turn ball around
		v_vel.x = -v_vel.x;
		v_pos.x += v_vel.x * dt;
	}

	return;
}

// reset
void CBall::Reset() {
	// set velocity
	v_vel.x = 0.0f;
	v_vel.z = -1.7f;

	// set position
	v_pos.x = 0.09f;
	v_pos.z = 0.75f;
}

// draw ball
void CBall::Draw() {
	// color
	glColor3f(1.0f, 0.7f, 0.0f);
	glBegin(GL_QUADS);
		glVertex3f(v_pos.x, 0, v_pos.z);
		glVertex3f(v_pos.x, 0, v_pos.z + h);
		glVertex3f(v_pos.x + w, 0, v_pos.z + h);
		glVertex3f(v_pos.x + w, 0, v_pos.z);
	glEnd();
}

// player bat constructor
CPlayer::CPlayer() {
	// set position
	v_pos.x = 0;
	v_pos.z = 1.45;

	// set velocity
	v_vel.x = 0.4f;

	// set size
	w = 0.15f;
	h = 0.01f;
}

//reset bat
void CPlayer::Reset() {
	// set position
	v_pos.x = 0;
	v_pos.z = 1.45;

	// set velocity
	v_vel.x = 0.4f;
}

// move bat left
void CPlayer::MoveL(float dt, float xmin) {
	// move based on delta time
	v_pos.x += -v_vel.x * dt;

	// check bounds
	if(v_pos.x < xmin) { v_pos.x = xmin; }
}

// move bat right
void CPlayer::MoveR(float dt, float xmax) {
	// move based on delta time
	v_pos.x += v_vel.x * dt;

	// check bounds
	if(v_pos.x > xmax - w) { v_pos.x = xmax - w; }
}

// draw player bat
void CPlayer::Draw() {
	// color
	glColor3f(1.0f, 1.0f, 0.0f);
	glBegin(GL_QUADS);
		glVertex3f(v_pos.x, 0, v_pos.z);
		glVertex3f(v_pos.x, 0, v_pos.z + h);
		glVertex3f(v_pos.x + w, 0, v_pos.z + h);
		glVertex3f(v_pos.x + w, 0, v_pos.z);
	glEnd();
}

// computer bat constructor
CComputer::CComputer() {
	// set position
	v_pos.x = 0;
	v_pos.z = -0.45;

	// set velocity
	v_vel.x = 0.4f;

	// set size
	w = 0.12f;
	h = 0.01f;
}

// reset bat
void CComputer::Reset() {
	// set position
	v_pos.x = 0;
	v_pos.z = -0.45;

	// set velocity
	v_vel.x = 0.35f;
}

// update bat
void CComputer::Update(float dt, float x1, float x2, CBall *ball) {
	// center of bat
	float center = v_pos.x + (w / 2);

	// move center of bat towards ball
	if(center < ball->v_pos.x) {
		// move bat
		v_pos.x += v_vel.x * dt;
	}
	if(center > ball->v_pos.x) {
		// move bat
		v_pos.x += -v_vel.x * dt;
	}

	// check arena bounds
	if(v_pos.x < x1) { v_pos.x = x1; }
	if(v_pos.x > x2 - w) { v_pos.x = x2 - w; }

	return;
}

// draw computer bat
void CComputer::Draw() {
	// color
	glColor3f(0.0f, 1.0f, 1.0f);
	glBegin(GL_QUADS);
		glVertex3f(v_pos.x, 0, v_pos.z);
		glVertex3f(v_pos.x, 0, v_pos.z + h);
		glVertex3f(v_pos.x + w, 0, v_pos.z + h);
		glVertex3f(v_pos.x + w, 0, v_pos.z);
	glEnd();
}

// check for collision
bool Collision(CGameOBJ *o1, CGameOBJ *o2) {
	// coordinates of reduced rectangles
	float l1, l2;
	float r1, r2;
	float t1, t2;
	float b1, b2;

	// left
	l1 = o1->v_pos.x;
	l2 = o2->v_pos.x;
	// top
	t1 = o1->v_pos.z;
	t2 = o2->v_pos.z;
	// right
	r1 = o1->v_pos.x + o1->w;
	r2 = o2->v_pos.x + o2->w;
	// bottom
	b1 = o1->v_pos.z + o1->h;
	b2 = o2->v_pos.z + o2->h;
	
	// check
	if(b1 < t2) return false;
	if(t1 > b2) return false;
	if(r1 < l2) return false;
	if(l1 > r2) return false;
	
	return true;
}
